﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.IO;
using HSDAQNet;
using System.Text.RegularExpressions;

namespace PETAR400_AI_Continue_Mode
{
    class Program
    {
        public static bool IsIP(string ip)  //Check the IP format
        {
            return Regex.IsMatch(ip, @"^((2[0-4]\d|25[0-5]|[01]?\d\d?)\.){3}(2[0-4]\d|25[0-5]|[01]?\d\d?)$");
        }

        static void Main(string[] args)
        {
            bool quit = false, ret = false;
            string IP = "";
            IntPtr hHS = IntPtr.Zero;
            uint totalcnt = 0;

            Console.WriteLine("!!This program demonstrates how to use the continuous acquisition AI Buffer mode!!");
            Console.WriteLine("PET-AR400 has 4 AI channel");
            while (!quit)
            {
                Console.WriteLine("Please Enter IP!!");
                IP = Console.ReadLine();
                ret = IsIP(IP);
                if (ret == false)
                {
                    Console.WriteLine("Please enter the correct IP");
                }
                else
                {
                    quit = true;
                    Thread.Sleep(100);
                }
            }
            quit = false;
            hHS = HSDAQNet.Sys.HS_Device_Create(IP);
            if (hHS == null || hHS == IntPtr.Zero)
            {
                Console.WriteLine("Connect Fail");
                Console.WriteLine("\nPress any key to close...");
                Console.ReadKey();
                return;
            }
            string ModuleName = HSDAQNet.Sys.HS_GetModelName(hHS);
            Console.WriteLine("Module Name      : " + ModuleName);
            Console.WriteLine("Firmware Version : " + HSDAQNet.Sys.GetFirmwareVersion(hHS));
            Console.WriteLine("HSDAQ Version    : V." + HSDAQNet.Sys.GetSDKVersion());
            Console.WriteLine("HSDAQNet Version : V." + HSDAQNet.Sys.GetHSDAQNetVersion());
            /* For details, please refer to the API manual
             * AI CH input Range : 1 ~ 4
             * Gain Range        : 0
             * Mode Range        : 0
             * Sample rate       : 
             *  ch=1    : 12.8k/16k/32k/64k/128k Hz
             *  ch=2    : 12.8k/16k/32k/64k Hz
             *  ch=3/4  : 12.8k/16k/32k Hz
             * targetcnt         : 0 (continue mode)
            */
            short ch = 4, gain = 0, mode = 0, DTM = 0, Autorun = 0;
            int samplerate = 32000;
            uint targetcnt = 0;
            HSDAQNet.HSIO.HS_SetAIScanParam(hHS, ch, gain, mode, samplerate, targetcnt, DTM, Autorun);

            HSDAQNet.HSIO.HS_GetAIScanParam(hHS, ref ch, ref gain, ref mode, ref samplerate, ref targetcnt, ref DTM, ref Autorun);
            Console.WriteLine("Channel    : " + ch);
            Console.WriteLine("Gain       : " + gain);
            Console.WriteLine("Mode       : " + mode);
            Console.WriteLine("Samplerate : " + samplerate);
            Console.WriteLine("targetcnt  : " + targetcnt);
            Console.WriteLine("---------------------------------------------------------");
            Console.WriteLine("Start AI scan!");
            HSDAQNet.HSIO.HS_StartAIScan(hHS);
            while (!quit)
            {
                ushort BufStatus = 0;
                uint BufferCnt = 0;
                HSDAQNet.HSIO.HS_GetAIBufferStatus(hHS, ref BufStatus, ref BufferCnt);
                if ((BufStatus & 0x02) == 0x02 || (BufStatus & 0x08) == 0x08)
                {
                    Console.WriteLine("Error code 0x" + HSDAQNet.ErrHandling.GetLastError().ToString("x8"));
                    HSDAQNet.HSIO.HS_StopAIScan(hHS);
                    break;
                }
                float[] fdatabuf = new float[BufferCnt];
                totalcnt += HSDAQNet.HSIO.HS_GetAIBuffer(hHS, fdatabuf, BufferCnt);
                Console.Write(totalcnt + "\r");

                if (Console.KeyAvailable)
                {
                    if (Console.ReadKey(true).Key == ConsoleKey.Q)
                    {
                        HSDAQNet.HSIO.HS_StopAIScan(hHS);
                        quit = true;
                    }
                }
            }
            HSDAQNet.Sys.HS_Device_Release(hHS);
            Console.WriteLine("\nTotal reads : " + totalcnt);
            Console.WriteLine("\nPress any key to close...");
            Console.ReadKey();
        }
    }
}
